import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * @author hobit
 *
 */
public class TextsToBin {
	public static final String IN_NAME = "texts.txt";
	public static final String OUT_NAME = "texts.bin";
		
	public void processTexts(String inName, String outName){
		Hashtable texts = readPlainTexts(inName);
		System.out.println(texts);
		saveBinTexts(texts, outName);
	}
	
	public void processTexts(){
		processTexts(IN_NAME, OUT_NAME);
	}
	
	public Hashtable readPlainTexts(String inName){
		Hashtable texts = new Hashtable();
		InputStream is = null;
		InputStreamReader isr = null;
		try {
			is = new FileInputStream(inName);
			isr = new InputStreamReader(is, "UTF-8");
			int i = isr.read();
			StringBuffer buf = new StringBuffer();
			String key = null;
			while (i != -1) {
				char ch = (char) i;
				if (ch != '\n' && (ch != ':' || key != null)) {
					// natn kle 
					buf.append(ch);
				} else if (ch == ':' && key == null) {
					key = buf.toString();
					buf.delete(0, buf.length());
				} else if (ch == '\n') {
					if (key != null) {
						texts.put(key, buf.toString());
						key = null;
					}
					buf.delete(0, buf.length());
				}
				i = isr.read();
			}
			if(key!=null){
				texts.put(key, buf.toString());
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if(isr!=null){
				try{
					isr.close();
				} catch(Exception e){
				}
			}
			if(is!=null){
				try{
					is.close();
				} catch(Exception e){
				}
			}			
		}
		return texts;
	}
	
	protected void saveBinTexts(Hashtable texts, String outName){
		FileOutputStream fos = null;
		DataOutputStream dos = null;
		try{
			// vytvoen vstupnho proudu do souboru
			fos = new FileOutputStream(outName);
			// vytvoen datovho vstupnho proudu,
			// kter data posl dl do
			// souborovho vstupnho proudu
			dos = new DataOutputStream(fos);
			Enumeration enum = texts.keys();
			// zapsn potu text
			dos.writeInt(texts.size());
			while(enum.hasMoreElements()){
				// uloen kle a pak hodnoty textu
				String key = (String)enum.nextElement();
				dos.writeUTF(key);
				dos.writeUTF((String)texts.get(key));
			}
		} catch(Exception e){
			e.printStackTrace();
		} finally {
			if(dos!=null){
				// zaven datovho vstupnho proudu
				try{
					dos.close();
				} catch(Exception e){
				}
			}
			if(fos!=null){
				// zaven souborovho vstupnho proudu
				try{
					fos.close();
				} catch(Exception e){
				}
			}
			
		}
		
	}

	/**
	 * Neobdr-li program dn argument, pouije jako vstupn soubor texts.txt
	 * a jako vstupn soubor texts.bin
	 * Obdr-li jeden argument, pouije jej jako vstupn soubor, jako vstupn
	 * soubor pouije texts.bin
	 * Obdr-li program dva argumenty, pouije prvn jako vstupn soubor a
	 * druh jako vstupn soubor 
	 */
	public static void main(String[] args) {
		TextsToBin ttb = new TextsToBin();
		if(args.length==0){
			ttb.processTexts();
		} else if (args.length == 1) {
			ttb.processTexts(args[0], TextsToBin.OUT_NAME);
		} else {
			ttb.processTexts(args[0], args[1]);
		}
	}
}
