
import java.io.DataInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Hashtable;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

/*
 * Created on 7.8.2004
 */

/**
 * @author Lucie Rt Bittnerov
 */
public class GameMIDlet extends MIDlet implements CommandListener {

	public static final String BACK = "back";
	public static final String HELP = "help";
	public static final String HELP_CONTENT = "help-content";
	public static final String PLAY = "play";
	public static final String EXIT = "exit";

	private GameCanvas canvas;
	private Displayable menu;
	private Displayable help;
	private boolean inited = false;

	private static GameMIDlet instance;

	// texty aplikace naten z externho souboru
	private Hashtable texts;

	public Command backCmd;

	public GameMIDlet() {
		super();
		instance = this;
		texts = readBinaryTexts();
		backCmd = new Command(getText(BACK), Command.BACK, 1);
	}
	
	/**
	 * Naten text z binrnho souboru
	 */
	protected Hashtable readBinaryTexts(){
		Hashtable texts = new Hashtable();
		InputStream is = null;
		DataInputStream dis = null;
		try {
			// vytvoen vstupnho proudu ze souboru
			is = this.getClass().getResourceAsStream("/texts.bin");
			// vytvoen datovho vstupnho proudu, kter
			// te data z ve vytvoenho vstupnho proudu
			dis = new DataInputStream(is);
			// naten potu text
			int i = dis.readInt();
			while(i>0){
				// naten textu
				texts.put(dis.readUTF(), dis.readUTF());
				i--;
			}
		} catch (Exception e) {
		} finally {
			if(dis!=null){
				// zaven datovho vstupnho proudu
				try{
					dis.close();
				} catch(Exception e){
				}
			}
			if(is!=null){
				// zaven vstupnho proudu
				try{
					is.close();
				} catch(Exception e){
				}
			}			
		}
		return texts;
	}
	
	/**
	 * Nate z textovho souboru texty. Oekvan formt souboru je ve tvaru
	 * kl: hodnota, piem co dek, to jeden text
	 * @return Hashtable obsahujc texty
	 */
	protected Hashtable readPlainTexts(){
		Hashtable texts = new Hashtable();
		InputStream is = null;
		InputStreamReader isr = null;
		try {
			// oteven zdrojovho souboru
			is = this.getClass().getResourceAsStream("/texts.txt");
			// vytvoen instance InputStreamReader se zadanm kdovnm UTF-8
			isr = new InputStreamReader(is, "UTF-8");
			int i = isr.read();
			StringBuffer buf = new StringBuffer();
			String key = null;
			while (i != -1) {
				// zpracovn dat, dokud nenastane konec souboru
				char ch = (char) i;
				if (ch != '\n' && (ch != ':' || key != null)) {
					// natn kle nebo jeho hodnoty
					buf.append(ch);
				} else if (ch == ':' && key == null) {
					// zpracovn dvojteky oddlujc kl
					// od jeho hodnoty
					key = buf.toString();
					buf.delete(0, buf.length());
				} else if (ch == '\n') {
					// zpracovn konce dku
					if (key != null) {
						texts.put(key, buf.toString());
						key = null;
					}
					buf.delete(0, buf.length());
				}
				// naten dalho znaku
				i = isr.read();
			}
			// nen-li posledn dek ukonen znakem novho dku
			// je poteba uloit posledn text zde
			if(key!=null){
				texts.put(key, buf.toString());
			}
		} catch (Exception e) {		
		} finally {
			if(isr!=null){
				// zaven instance InputStreamReader
				try{
					isr.close();
				} catch(Exception e){
				}
			}
			if(is!=null){
				// zaven instance InputStream
				try{
					is.close();
				} catch(Exception e){
				}
			}			
		}
		return texts;
	}

	/**
	 * @return vrt instanci tdy GameMIDlet
	 */
	public static GameMIDlet getInstance() {
		return instance;
	}

	protected void startApp() throws MIDletStateChangeException {
		if (canvas == null) {
			// Sputn aplikace
			SplashScreen splashScreen = new SplashScreen();
			Display.getDisplay(this).setCurrent(splashScreen);
			Thread t = new Thread(splashScreen);
			t.start();
			canvas = new GameCanvas(this);
			// zskn jmna implementace menu z deskriptoru aplikace
			String className = this.getAppProperty("menu-class");
			if (className == null) {
				className = "MenuList";
			}
			try {
				// vytvoen instance tdy se jmnem className
				menu = (Displayable) Class.forName(className).newInstance();
			} catch (Exception e) {
			}
			//menu = new MenuList(this);
			initHelp();
			inited = true;
		} else {
			// Aktivace aplikace po jejm pobytu v pasivnm stavu,
			// v naem ppad zde nen nic poteba dlat
		}
	}

	/**
	 * @return true, je-li instance inicializovan, jinak false
	 */
	public boolean isInited() {
		return inited;
	}

	protected void pauseApp() {
	}

	protected void destroyApp(boolean arg0) {
		notifyDestroyed();
	}

	/**
	 * Zobraz vlastn hru
	 *
	 */
	public void playAction() {
		if (canvas.isOver()) {
			canvas.init();
		}
		Display.getDisplay(this).setCurrent(canvas);
	}

	/**
	 * Zobraz menu
	 *
	 */
	public void menuAction() {
		Display.getDisplay(this).setCurrent(menu);
	}

	/**
	 * Zobraz npovdu
	 *
	 */
	public void helpAction() {
		Display.getDisplay(this).setCurrent(help);
	}

	/**
	 * Inicializuje npovdu
	 *
	 */
	protected void initHelp() {
		Form help = new Form(getText(HELP));
		help.append(getText(HELP_CONTENT));
		help.addCommand(backCmd);
		help.setCommandListener(this);
		this.help = help;
	}

	public void commandAction(Command c, Displayable d) {
		if (d == help && c == backCmd) {
			menuAction();
		} else if (d == canvas && c == backCmd) {
			menuAction();
		}
	}

	/**
	 * @param key kl, kterm je text uren
	 * @return text odpovdajc danmu kli
	 */
	public String getText(String key) {
		return (String) texts.get(key);
	}
}